/*
 * Copyright 2008-2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.entropysoft.transmorph.converters;

import javax.xml.namespace.QName;

import net.entropysoft.transmorph.ConversionContext;
import net.entropysoft.transmorph.ConverterException;
import net.entropysoft.transmorph.type.TypeReference;

/**
 * Converter used when source object is a String and destination type is a
 * QName
 * 
 * @author Cedric Chabanois (cchabanois at gmail.com)
 * 
 */
public class StringToQName extends AbstractSimpleConverter<String, QName> {

	public StringToQName() {
		super(String.class, QName.class);
		this.useObjectPool = true; // immutable
	}

	@Override
	public QName doConvert(ConversionContext context, String sourceObject, TypeReference<?> destinationType) throws ConverterException {
		if (sourceObject == null) {
			return null;
		}
		return QName.valueOf(sourceObject);
	}

}
